-- Preset3D --

function d3.preset.triPyramid( sideLength, ox, oy, oz )
	local s = sideLength/2
	rt3, rt6 = math.sqrt(3), math.sqrt(6)
	local points = {
		{ s,  -s/rt3, -s/rt6},
		{-s,  -s/rt3, -s/rt6},
		{ 0, 2*s/rt3, -s/rt6},
		{ 0,       0, 3*s/rt6}}
	for i = 1, #points do
		points[i][1] = points[i][1] + ox
		points[i][2] = points[i][2] + oy
		points[i][3] = points[i][3] + oz
		end
	local polyTable = {						-- Right-hand rule!
	{1, 2, 3, color = {255,  0,  0}},
	{3, 2, 4, color = {  0,255,  0}},
	{1, 4, 2, color = {255,255,  0}},
	{1, 3, 4, color = {  0,  0,255}}
	}
	return points, polyTable
	end

function d3.preset.Sphere( sideLength, ox, oy, oz )
	local s = sideLength/2
	local s3 = s*math.sqrt(3)--4*s--s*math.sqrt(3)
	rt3, rt6 = math.sqrt(3), math.sqrt(6)
	points = {
		{ s,  s,  s},
		{ s,  s, -s},
		{ s, -s, -s},
		{ s, -s,  s},
		{-s, -s,  s},
		{-s,  s,  s},
		{-s,  s, -s},
		{-s, -s, -s},
		{s3,  0,  0},
		{ 0,  0, s3},
		{-s3, 0,  0},
		{ 0,  0,-s3},
		{ 0, s3,  0},
		{ 0,-s3,  0}}
	for i = 1, #points do
		points[i][1] = points[i][1] + ox
		points[i][2] = points[i][2] + oy
		points[i][3] = points[i][3] + oz
		end
	local polyTable = {						-- right-hand rule!
	{2, 1,  9, color = {  0,255,255}},
	{3, 2,  9, color = {  0,255,255}},
	{4, 3,  9, color = {  0,255,255}},
	{1, 4,  9, color = {  0,255,255}},
	{4, 1, 10, color = {255,  0,255}},
	{1, 6, 10, color = {255,  0,255}},
	{6, 5, 10, color = {255,  0,255}},
	{5, 4, 10, color = {255,  0,255}},
	{5, 6, 11, color = {  0,  0,255}},
	{6, 7, 11, color = {  0,  0,255}},
	{7, 8, 11, color = {  0,  0,255}},
	{8, 5, 11, color = {  0,  0,255}},
	{3, 8, 12, color = {255,  0,  0}},
	{8, 7, 12, color = {255,  0,  0}},
	{7, 2, 12, color = {255,  0,  0}},
	{2,	3, 12, color = {255,  0,  0}},
	{1, 2, 13, color = {255,255,  0}},
	{2, 7, 13, color = {255,255,  0}},
	{7, 6, 13, color = {255,255,  0}},
	{6, 1, 13, color = {255,255,  0}},
	{3, 4, 14, color = {  0,255,  0}},
	{8, 3, 14, color = {  0,255,  0}},
	{5, 8, 14, color = {  0,255,  0}},
	{4, 5, 14, color = {  0,255,  0}}
	}
	return points, polyTable
	end

function d3.preset.Star( sideLength, ox, oy, oz )
	local s = sideLength/2
	local s3 = s*math.sqrt(3)--4*s--s*math.sqrt(3)
	rt3, rt6 = math.sqrt(3), math.sqrt(6)
	local points = {
			{     0,	  s,      0},
			{ .24*s,  .32*s,      0},
			{ .95*s,  .31*s,      0},
			{ .38*s, -.12*s,      0},
			{ .59*s, -.81*s,      0},
			{     0, -.40*s,      0},
			{-.59*s, -.81*s,      0},
			{-.38*s, -.12*s,      0},
			{-.95*s,  .31*s,      0},
			{-.24*s,  .32*s,      0},
			{     0,      0,  .25*s},
			{     0,      0, -.25*s}}
	for i = 1, #points do
		points[i][1] = points[i][1] + ox
		points[i][2] = points[i][2] + oy
		points[i][3] = points[i][3] + oz
		end

	local polyTable = {						-- right-hand rule!
	{2, 1, 11, color = {255,0,155}},
	{3, 2, 11, color = {155,0,255}},
	{4, 3, 11, color = {255,0,155}},
	{5, 4, 11, color = {155,0,255}},
	{6, 5, 11, color = {255,0,155}},
	{7, 6, 11, color = {155,0,255}},
	{8, 7, 11, color = {255,0,155}},
	{9, 8, 11, color = {155,0,255}},
	{10,9, 11, color = {255,0,155}},
	{1,10, 11, color = {155,0,255}},
	{1, 2, 12, color = {255,0,155}},
	{2, 3, 12, color = {155,0,255}},
	{3, 4, 12, color = {255,0,155}},
	{4, 5, 12, color = {155,0,255}},
	{5, 6, 12, color = {255,0,155}},
	{6, 7, 12, color = {155,0,255}},
	{7, 8, 12, color = {255,0,155}},
	{8,	9, 12, color = {155,0,255}},
	{9,10, 12, color = {255,0,155}},
	{10,1, 12, color = {155,0,255}}
	}
	return points, polyTable
	end

function d3.preset.triCube3D( sideLength, ox, oy, oz )
	local s = sideLength/2
	local points = {
		{-s, -s, -s},
		{ s, -s, -s},
		{ s,  s, -s},
		{-s,  s, -s},
		{-s, -s,  s},
		{ s, -s,  s},
		{ s,  s,  s},
		{-s,  s,  s}}
	for i = 1,#points do
		points[i][1] = points[i][1] + ox
		points[i][2] = points[i][2] + oy
		points[i][3] = points[i][3] + oz
		end
	local polyTable = {				-- converted to right-hand rule!
		{1, 3, 2, color = {255,  0,  0}},
		{1, 4, 3, color = {255,  0,  0}},
		{1, 2, 6, color = {  0,255,  0}},
		{1, 6, 5, color = {  0,255,  0}},
		{5, 6, 7, color = {  0,  0,255}},
		{5, 7, 8, color = {  0,  0,255}},
		{3, 4, 7, color = {225,225,  0}},
		{4, 8, 7, color = {255,255,  0}},
		{2, 7, 6, color = {255,  0,255}},
		{2, 3, 7, color = {255,  0,255}},
		{5, 4, 1, color = {  0,255,255}},
		{5, 8, 4, color = {  0,255,255}}
		}
	return points, polyTable
	end

function d3.preset.square( sideLength, ox, oy, oz )
	local s = sideLength/2
	local points = {
		{-s, -s, 0},
		{ s, -s, 0},
		{ s,  s, 0},
		{-s,  s, 0}}
	for i = 1,#points do
		points[i][1] = points[i][1] + ox
		points[i][2] = points[i][2] + oy
		points[i][3] = points[i][3] + oz
		end
	local polyTable = {
	{1, 2, 3, color = {255,  0,  0}},
	{1, 3, 4, color = {255,255,  0}},
	{1, 3, 2, color = {  0,255,  0}},
	{1, 4, 3, color = {  0,  0,255}}}
	return points, polyTable
end

function reverseHandedness( poly )
	for i = 1,#poly do poly[i][1], poly[i][2] = poly[i][2], poly[i][1] end
	return poly
	end
