--[[
The Ref3D Library --
License : Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported --
http://creativecommons.org/licenses/by-nc-sa/3.0/ --
Programmed by substitute541
This is NOT the L3D Library. the L3D Library was made by 10$man
This is an unauthorized (personal) version of substitute541's library
]]--

-- Settings
local Ref3D	= {}
Ref3D.graphics = {}
Ref3D.func		= {}
Ref3D.light	= {}
Ref3D.preset	= {}
Ref3D.version	= "0.3.2.mod"
sin, cos = math.sin, math.cos

function Ref3D.load( fl, vpX, vpY )
	focalLength = fl
	vanishingPointX, vanishingPointY = vpX, vpY
	require "Light3D"
	require "Preset3D"
	zSortUsingMinZ		= false
	isUsingBackfaceCull = true
	end

function Ref3D.calculatePointPosition( pt )	-- 3D => 2D
	local scale	= focalLength/(focalLength + pt[3])
	local newX	= vanishingPointX + pt[1] * scale
	local newY	= vanishingPointY + pt[2] * scale
	return {newX, newY}
	end

-- Check for backface
function Ref3D.isBackFace( p1, p2, p3 )
	local cx = p3[1] - p1[1]
	local cy = p3[2] - p1[2]
	local bx = p2[1] - p3[1]
	local by = p2[2] - p3[2]
	return cx * by >= cy * bx	-- <= change (equal added)
	end

-- Draws a 2D Circle
function Ref3D.graphics.circle2D(mode, lgt, radius, segments)
	local scale	= focalLength / ( focalLength + lgt[3] )
	local New	= Ref3D.calculatePointPosition(lgt)
	love.graphics.circle( mode, New[1], New[2], radius * scale, segments )
	end

-- Draws a triangle
function Ref3D.graphics.triangle( mode, bfc, pt1, pt2, pt3, ox, oy, sz )
	local New1 = Ref3D.calculatePointPosition( pt1 )
	local New2 = Ref3D.calculatePointPosition( pt2 )
	local New3 = Ref3D.calculatePointPosition( pt3 )
	if bfc then											-- back facing culling
		if Ref3D.isBackFace( New1, New2, New3 ) then
			return										-- (don't draw back facing)
			end
		end
	love.graphics.triangle(mode,
		sz * ( New1[1] - vp.x ) +vp.x + ox, sz * ( New1[2] - vp.y ) + vp.y + oy,
		sz * ( New2[1] - vp.x ) +vp.x + ox, sz * ( New2[2] - vp.y ) + vp.y + oy,
		sz * ( New3[1] - vp.x ) +vp.x + ox, sz * ( New3[2] - vp.y ) + vp.y + oy)
	end

-- Rotates a certain point, by default, around the vanishing points
function Ref3D.func.rotateX(y, z, angleX, centerY, centerZ)
	local y1 = (y - centerY) * cos(angleX) - (z - centerZ) * sin(angleX)
	local z1 = (z - centerZ) * cos(angleX) + (y - centerY) * sin(angleX)
	return y1 + centerY, z1 + centerZ
	end

function Ref3D.func.rotateY(x, z, angleY, centerX, centerZ)
	local x1 = (x - centerX) * cos(angleY) - (z - centerZ) * sin(angleY)
	local z1 = (z - centerZ) * cos(angleY) + (x - centerX) * sin(angleY)
	return x1 + centerX, z1 + centerZ
	end

function Ref3D.func.rotateZ(x, y, angleZ, centerX, centerY)
	local x1 = (x - centerX) * cos(angleZ) - (y - centerY) * sin(angleZ)
	local y1 = (y - centerY) * cos(angleZ) + (x - centerX) * sin(angleZ)
	return x1 + centerX, y1 + centerY
	end

-- Finds the average Z
function Ref3D.func.getAvgZTriangle(az, bz, cz)
	local avgZ = (az + bz + cz)/3
	return avgZ
	end

-- Sorts by z order
function Ref3D.func.zSortTriangle( poly, pts )	-- if zSortUsingMinZ = false
	for i = 1, #poly do
		poly[i].avgZ = Ref3D.func.getAvgZTriangle(pts[poly[i][1]][3], pts[poly[i][2]][3], pts[poly[i][3]][3])
		end
	table.sort(poly, function(A, B) return A.avgZ > B.avgZ end)
	end

function Ref3D.func.zSortTriangle2( poly, pts )	-- if zSortUsingMinZ = true
	for _, p in ipairs(poly) do
		p.minZ = math.min(pts[p[1]][3], pts[p[2]][3], pts[p[3]][3])
		end
	table.sort(poly, function(A, B) return A.minZ > B.minZ end)
	end
function Ref3D.loadData(name)
print(name)
	local points2, polygons2 = {},{}
	local mode = 'none'
	for line in love.filesystem.lines( name ) do
		if line == 'POINTS' then mode = 1
			elseif line == 'POLY' then mode = 2
			else
				if mode ==1 then table.insert(points2,line:split())
					elseif mode == 2 then table.insert(polygons2,line:split())
					end
			end
		end
	for i = 1, #polygons2 do
		polygons2[i][1] = tonumber(polygons2[i][1])
		polygons2[i][2] = tonumber(polygons2[i][2])
		polygons2[i][3] = tonumber(polygons2[i][3])
		polygons2[i].color = {}
		end
	return points2, polygons2
	end

function loadData(filename)
	local points2, polygons2 = {},{}
	local mode = 'none'
	for line in love.filesystem.lines( filename ) do
		if line == 'POINTS' then mode = 1
			elseif line == 'POLY' then mode = 2
			elseif line == 'NORM' then mode = 3
			else
				if mode ==1 then table.insert(points2,line:split())
					elseif mode == 2 then table.insert(polygons2,line:split())
					elseif mode == 3 then stop = true
					end
			end
		end

	for i = 1, #polygons2 do
		polygons2[i][1] = polygons2[i][1]+1
		polygons2[i][2] = polygons2[i][2]+1
		polygons2[i][3] = polygons2[i][3]+1
		polygons2[i].color = {}
		end
	s = 2.5
	for i = 1, #points2 do
		points2[i][1] = s * points2[i][1]
		points2[i][2] = s * points2[i][2]
		points2[i][3] = s * points2[i][3]
		end
	return points2, polygons2
	end

function Ref3D.sight()
	if zoom < 1.5 then gr.setColor(0,255,0,255) elseif
		zoom <2 then gr.setColor(255,255,0,255)
		else gr.setColor(255,0,0,255) end
	r,cx,cy= 20*zoom,screenWidth/2,screenHeight/2
	gr.circle('line',cx,cy,r,r)
	gr.line(cx-2*r,cy,cx-r,cy)
	gr.line(cx+2*r,cy,cx+r,cy)
	gr.line(cx,cy-2*r,cx,cy-r)
	gr.line(cx,cy+2*r,cx,cy+r)
	gr.line(cx-4*r,cy-3*r,cx-3*r,cy-3*r)
	gr.line(cx-4*r,cy-3*r,cx-4*r,cy-2*r)
	gr.line(cx+4*r,cy-3*r,cx+3*r,cy-3*r)
	gr.line(cx+4*r,cy-3*r,cx+4*r,cy-2*r)

	gr.line(cx-4*r,cy+3*r,cx-3*r,cy+3*r)
	gr.line(cx-4*r,cy+3*r,cx-4*r,cy+2*r)
	gr.line(cx+4*r,cy+3*r,cx+3*r,cy+3*r)
	gr.line(cx+4*r,cy+3*r,cx+4*r,cy+2*r)
	end

function string:split(sep)
		local sep, fields = sep or ",", {}
		local pattern = string.format("([^%s]+)", sep)
		self:gsub(pattern, function(c) fields[#fields+1] = c end)
		return fields
		end

return Ref3D
